#!/usr/bin/env python3
# vim: set ft=python fileencoding=utf-8 tw=72:
"""
A blog-oriented static web content generator from Markdown to HTML5.

    :author: J A Corbal
    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""

from pynfact.cli import main


if __name__ == '__main__':
    main()




# vim: set ft=python fileencoding=utf-8 tw=72 fdm=indent nowrap:
"""
File and path manipulation functions.

    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""

from pynfact.struri import slugify, strip_html_tags
import os


def link_to(name, prefix='', makedirs=True, justdir=False,
            index='index.html'):
    """Make a link relative path in terms of the build.

    It may return a string to the destination, and also create that path
    if it doesn't exist. 

    :param name: Filename (extension will be disregarded)
    :type name: str
    :param prefix: Prefix directory to preppend the file
    :type prefix: str
    :param makedirs: If true make all needed directories
    :type makedirs: bool
    :param justdir: If true return only the link dir., not the full path
    :type justdir: bool
    :param index: Default name of file contained in the path
    :type index: str
    :return: Link path to a  ``index.html`` page
    :rtype: str

    :Example:

    >>> struri.link_to('dest', '/a/b/c', makedirs=False)
    '/a/b/c/dest/index.html'

    >>> struri.link_to('dest', 'a/b/c', makedirs=False, justdir=True)
    'a/b/c/dest'

    .. versionchanged:: 1.2.0a1, changed function location from module
                        ``struri`` to ``fileman``.

    .. todo:: Add a verbose argument, default to ``False`` (for
              compatibility), that prints status on creating the path if
              ``makedirs=True``.
    """
    dirname = slugify(strip_html_tags(os.path.splitext(name)[0]))
    path = os.path.join(prefix, slugify(dirname), index)
    if makedirs:
        os.makedirs(os.path.dirname(path), exist_ok=True)

    return os.path.dirname(path) if justdir else path


def has_extension(filename, extension):
    """Test if the filename has a extension in particular.

    :param filename: Filename to test
    :type filname: str
    :param extension: Extension to test against
    :type extension: str
    :return: True if the filename has the inquired
    :rtype: bool
    """
    return os.path.splitext(filename)[1] == extension




# vim: set ft=python fileencoding=utf-8 tw=72 fdm=indent nowrap:
"""
URI strings manipulation functions.

    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""
import unidecode
import re


def slugify(unslugged, separator='-'):
    """Slug a string.

    :param unslugged: String to slugify
    :type unslugged: str
    :param separator: Character used to separate words
    :type separator: str
    :return: Slugged string
    :rtype: str
    """
    return re.sub(r'\W+', separator,
                  unidecode.unidecode(unslugged).strip().lower())


def strip_html_tags(text):
    """Strip HTML tags in a string.

    :param text: String containing HTML code
    :type text: str
    :return: String without HTML tags
    :rtype: str
    """
    return re.sub('<[^<]+?>', '', text)




# vim: set ft=python fileencoding=utf-8 tw=72 fdm=indent nowrap:
"""
Markdown translation to HTML and metadata "getter".

    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""
import markdown
import textwrap


class Mulang:
    """Generate HTML code from MarkUp LANGuage (mulang) source."""

    def __init__(self, input_data, encoding='utf-8',
                 output_format='html5', logger=None):
        """Constructor.

        :param input_data: File from where the data is taken
        :type input_data: str
        :param encoding: Encoding the input file is in
        :type encoding: str
        :param output_format: Output HTML version
        :type output: str
        :param logger: Logger where to store activity in
        :type logger: logging.Logger

        This class has methods to parse the markup language and get the
        information.  The logging activity is only set to debug, since
        it's not neccessary to tell continously the default activity.
        """
        self.input_data = input_data
        self.encoding = encoding
        self.md = markdown.Markdown(
            extensions=['markdown.extensions.extra',
                        'markdown.extensions.toc',
                        'markdown.extensions.abbr',
                        'markdown.extensions.def_list',
                        'markdown.extensions.footnotes',
                        'markdown.extensions.codehilite',
                        'markdown.extensions.meta'],
            encoding=encoding,
            output_format=output_format)
        self.logger = logger

    def html(self):
        """Generate HTML from Markdown data."""
        input_file = open(self.input_data, mode="r",
                          encoding=self.encoding)
        text = input_file.read()
        input_file.close()
        html = self.md.convert(text)

        self.logger and self.logger.debug('Parsed body data of: "%s"',
                                          self.input_data)

        return html

    def metadata(self):
        """Fetch metadata in the markdown file.

        .. todo: Get the meta without generating HTML again
        """
        input_file = open(self.input_data, mode="r",
                          encoding=self.encoding)
        text = input_file.read()
        input_file.close()
        html = self.md.convert(text)

        self.logger and self.logger.debug('Parsed meta data of: "%s"',
                                          self.input_data)

        return self.md.Meta




# vim: set ft=python fileencoding=utf-8 tw=72 fdm=indent nowrap:
"""
Meta information retrieval from markdown post file.

Meta fields are set in English, Portuguese, Spanish, Catalan, Galician
and Esperanto (using the *X*-system), although there cannot be any
accent in the field names, so "título" has to be "titulo" in order to
work properly.

There may be diacritics in field values (UTF-8 by default) but not in
field names.

    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""
from dateutil.parser import parse
import datetime
import markdown
import re


class Meta:
    """Meta information retriever.

    .. todo: Use a general function retriever, instead of one function
             for each meta key.

    .. code-block:

        def retrieve(self, array_of_valid_keys,
                type=[str, list, joined, date],
                date_format[if_type is date]):

    * ``type`` is ``date``, use the `date_format`
    * ``type`` is ``str``, return the trimmed string as it comes
    * ``type`` is ``joined``, return the list after splitting commas

    Code example of the intended feature::

        >>> m = Meta(meta_information)

        >>> title_valid_keys = ['title', 'titulo', 'titolo', 'titol' ]
        >>> def m.retrieve(title_valid_keys, type='str')
        'The title to this Post'

        >>> date_valid_keys = ['date', 'data', 'dato', 'fecha' ]
        >>> def m.retrieve(date_valid_keys, type='date', '%c')
        'Thu 12 Mar 2020 06:47:43 PM UTC'

        >>> tags_valid_keys = ['tags', 'labels', 'etiquetas', 'etikedoj']
        >>> def m.retrieve(tags_valid_keys, type='join')
        ['tag1', 'tag2', 'tag3', 'tag4']

    """

    def __init__(self, meta):
        """Constructor.

        :param meta: Meta information
        :type meta: list
        """
        self.meta = meta

    def category(self, default_category='Miscellaneous'):
        """Get category as a string from post meta information.

        :return: Category field
        :rtype: str
        """
        category = self.meta.get('category') or \
            self.meta.get('categoria') or self.meta.get('kategorio')
        return ' '.join(category) if category else default_category

    def author(self, default_author=''):
        """Get the author as a string from post meta information.

        .. todo:: If more than one author, return list, instead of str

        :return: Author field
        :rtype: str
        """
        authors = self.meta.get('authors') or self.meta.get('author') or \
            self.meta.get('autores') or self.meta.get('autors') or \
            self.meta.get('autor') or self.meta.get('autoro') or \
            self.meta.get('auxtoro')
        return ', '.join(authors) if authors else default_author

    def email(self):
        """Get the author's email as a string from post meta information.

        :return: Email field
        :rtype: str
        """
        emails = self.meta.get('email') or self.meta.get('e-mail') or \
            self.meta.get('correo electronico') or \
            self.meta.get('correo') or self.meta.get('correu') or \
            self.meta.get('e-correo') or \
            self.meta.get('retposto') or self.meta.get('retposxto')
        return ', '.join(emails) if emails else ''

    def language(self, default_language="en"):
        """Get the post language as a string from post meta information.

        :return: Language field
        :rtype: str
        """
        language = self.meta.get('language') or self.meta.get('idioma') or \
            self.meta.get('lengua') or self.meta.get('lingua') or \
            self.meta.get('llengua') or self.meta.get('lingvo')
        return language if language else default_language

    def title(self):
        """Get post title as a string from post meta information.

        :return: Title field
        :rtype: str
        """
        title = self.meta.get('title') or self.meta.get('titulo') or \
            self.meta.get('entry') or self.meta.get('entrada') or \
            self.meta.get('titol') or self.meta.get('post') or \
            self.meta.get('titolo')
        # return ''.join(title) if title else ''
        fmttitle = markdown.markdown(' '.join(title) if title else '')
        return re.sub(r'</*(p|br)[^>]*?>', '', fmttitle)

    def summary(self):
        """Get post summary as a string from post meta information.

        :return: Summary field
        :rtype: str
        """
        summary = self.meta.get('summary') or self.meta.get('resumo') or \
            self.meta.get('resumen') or self.meta.get('resum')
        fmtsummary = markdown.markdown(' '.join(summary) if summary else '')
        return re.sub(r'</*(p|br)[^>]*?>', '', fmtsummary)

    def copyright(self):
        """Get post copyright as a string from post meta information.

        :return: Copyright field
        :rtype: str
        """
        copyright = self.meta.get('copyright') or \
            self.meta.get('license') or \
            self.meta.get('licencia') or \
            self.meta.get('licenza') or \
            self.meta.get('llicencia') or \
            self.meta.get('permesilo') or \
            self.meta.get('kopirajto') or \
            self.meta.get('(c)')
        fmtcopyright = markdown.markdown(' '.join(copyright) if copyright
                                         else '')
        return re.sub(r'</*(p|br)[^>]*?>', '', fmtcopyright)

    def date_info(self):
        """Get post date as a datetime object.

        :return: Date field
        :rtype: datetime
        """
        date = self.meta.get('date') or self.meta.get('data') or \
            self.meta.get('fecha') or self.meta.get('dato')
        return parse(''.join(date)) if date else ''

    def date(self, date_format='%c'):
        """Get post date as a string.

        :param date_format: Date locale
        :type date_format: str
        :return: Date field
        :rtype: str
        """
        return self.date_info().strftime(date_format) \
            if self.date_info() else ''

    def up_date_info(self):
        """Get updated post date as a datetime object.

        :return: Date field
        :rtype: datetime
        """
        up_date = self.meta.get('updated') or \
            self.meta.get('actualizado') or \
            self.meta.get('actualitzat') or \
            self.meta.get('gisdatigita') or \
            self.meta.get('gxisdatigita')
        return parse(''.join(up_date)) if up_date else ''

    def up_date(self, date_format='%c'):
        """Get updated post date as a string.

        :param date_format: Date locale
        :type date_format: str
        :return: Date field
        :rtype: str
        """
        return self.up_date_info().strftime(date_format) \
            if self.up_date_info() else ''

    def comments(self):
        """Get comments status.

        :return: True if entry has comments, or False otherwise
        :rtype: bool
        """
        comments = self.meta.get('comments') or \
            self.meta.get('comentarios') or \
            self.meta.get('comentaris') or \
            self.meta.get('komentoj') or ['']
        com_bool = False if comments[0].lower() == "no" or \
            comments[0].lower() == "non" or \
            comments[0].lower() == "não" or \
            comments[0].lower() == "ne" \
            else True
        return com_bool

    def private(self):
        """Get private status.

        :return: True if entry is private, or False otherwise
        :rtype: bool
        """
        private = self.meta.get('private') or \
            self.meta.get('privado') or \
            self.meta.get('privat') or \
            self.meta.get('privata') or ['']
        piv_bool = True if private[0].lower() == "yes" or \
            private[0].lower() == "si" or \
            private[0].lower() == "sí" or \
            private[0].lower() == "sim" or \
            private[0].lower() == "jes" \
            else False
        return piv_bool

    def navigation(self):
        """Get navigation status.

        :return: True if entry is navigable, or False otherwise
        :rtype: bool
        """
        navigation = self.meta.get('nav') or \
            self.meta.get('navigation') or \
            self.meta.get('navegacion') or \
            self.meta.get('navegacio') or \
            self.meta.get('navigado') or ['']
        nav_bool = False if navigation[0].lower() == "no" or \
            navigation[0].lower() == "non" or \
            navigation[0].lower() == "não" or \
            navigation[0].lower() == "ne" \
            else True
        return nav_bool

    def tag_list(self):
        """Get the tags as a list from post meta information.

        :return: List of tags
        :rtype: list
        """
        tag_list = self.meta.get('tags') or self.meta.get('labels') or \
            self.meta.get('etiquetas') or \
            self.meta.get('etiquetes') or \
            self.meta.get('etikedoj') or ['']
        tag_list = [x.strip() for x in tag_list[0].split(',') if x.strip()]
        return tag_list if tag_list else []

    def tags(self):
        """Get the tags as a list from post meta information.

        :return: List of tags
        :rtype: str
        """
        return ', '.join(tag_list(self.meta)) if tag_list(self.meta) else ''




#!/usr/bin/env python3
# vim: set ft=python fileencoding=utf-8 tw=72:
#
########################################################################
# The MIT License (MIT)
#
# Copyright (c) 2012-2020, J. A. Corbal
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# “Software”), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
########################################################################
"""
A blog-oriented static web content generator from Markdown to HTML5.

    :author: J A Corbal
    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""
__author__ = "J. A. Corbal"
__copyright__ = "Copyright 2012-2020, J. A. Corbal"
__email__ = "jacorbal@gmail.com"
__license__ = "MIT"
__maintainer__ = "J. A. Corbal"
__status__ = "Production"
__version_info__ = ('1', '3', '1a2')
__version__ = '.'.join(__version_info__)




# vim: set ft=python fileencoding=utf-8 tw=72 fdm=indent nowrap:
"""
Handles a YAML file by setting a default value when variable is not set.

    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""
import sys
import yaml


class Yamler:
    """Handle a YAML file."""

    def __init__(self, filename, logger=None):
        """Constructor.

        :param filename: YAML filename where to look for the config.
        :type filename: str
        :raise: IOError
        """
        self.filename = filename
        self.logger = logger
        self.fd = None
        try:
            with open(self.filename, 'r') as self.fd:
                self.config = yaml.safe_load(self.fd)
        except IOError:
            self.logger and self.logger.error(
                "Cannot read the configuration file")
            sys.exit(21)

    def __del__(self):
        """Destructor.

        Close all opened files, just to make sure, although those will
        be automatically closed.
        """
        if (self.fd):
            self.fd.close()

    def __getitem__(self, key):
        """Get the value from a specific key in the configuration file.

        :param key: Key to look for
        :type key: str
        :return: The value associated to that key
        :raise: KeyError
        """
        try:
            value = self.config[key]
        except KeyError:
            self.logger and self.logger.error(
                "Key not found in configuration file")
            sys.exit(22)
        else:
            return value

    def retrieve(self, key, default_value=None):
        """Get a value from a key or sets a default value.

        :param key: Key to look for
        :type key: str
        :param default_value: Default value if key is not found
        :return: The value associated to that key, or the default value

        .. todo:: Use :func:`yamler.Yamler.__getitem__`?
        """
        if key in self.config:
            value = self.config[key]
        else:
            value = default_value
        return value




#!/usr/bin/env python3
# vim: set ft=python fileencoding=utf-8 tw=72 fdm=indent nowrap:
"""
Command line interface.

    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""
from pynfact.builder import Builder
from pynfact.server import Server
from pynfact.yamler import Yamler
import argparse
import logging
import os
import shutil
import sys


try:
    import colored_traceback.auto
    colored_traceback.add_hook(always=True)
except ImportError:
    pass


def set_logger(verbosity=False, error_log='pynfact.err',
               echo_log=sys.stdout):
    """Set up the system logger.

    :param verbosity: Show basic information, or also debug messages
    :type verbosity: bool
    :param error_log: Filename for the warnings and errors log
    :type error_log: str
    :param echo_log: Stream to write the default information log
    :type echo_log: _io.TextIOWrapper

    This function starts two logs, one stream on the standard output
    (``echo_log``) where the ``verbosity`` parameter can increase the
    volume of messages; and the error log (``error_log``), saved to a
    file but only logging errors and critical errors.  Warnings, errors
    and critical errors are displayed automatically to the standard
    output.

    The parameter ``verbosity`` concerns only to what is being displayed
    on screen concordingly with the log leves of the ``logging`` module.
    These levels may be specified by an integer value or by name.  These
    are their values:

    * 10: ``logging.DEBUG``
    * 20: ``logging.INFO``
    * 30: ``logging.WARNING``
    * 40: ``logging.ERROR``
    * 50: ``logging.CRITICAL``

    The following table illustrates what is getting logged:

    +---------------+--------------+--------------+---------------+
    | ``logging.``  | ``!verbose`` |  ``verbose`` | ``error_log`` |
    +===============+==============+==============+===============+
    | ``.DEBUG``    |              |      X       |               |
    +---------------+--------------+--------------+---------------+
    | ``.INFO``     |      X       |      X       |               |
    +---------------+--------------+--------------+---------------+
    | ``.WARNING``  |      X       |      X       |               |
    +---------------+--------------+--------------+---------------+
    | ``.ERROR``    |      X       |      X       |       X       |
    +---------------+--------------+--------------+---------------+
    | ``.CRITICAL`` |      X       |      X       |       X       |
    +---------------+--------------+--------------+---------------+
    | write log to: |  ``stdout``  |  ``stdout``  |   ``file``    |
    +---------------+--------------+--------------+---------------+

    The only way to deactivate the ``error_log`` is to use the command
    line option ``-l none``, or ``--log=none``.  Either way, the
    warnings, errors and critical messages will still be shown on
    screen, for any value of ``verbosity``.  What ``verbosity`` does is
    to enable or disabe the debug messages.
    """
    log_level = logging.DEBUG if verbosity else logging.INFO
    logger = logging.getLogger(__name__)
    logger.setLevel(log_level)

    # Default ``stdout`` handler, set by ``verbosity`` value
    echo_shandler = logging.StreamHandler(echo_log)
    echo_shandler.setLevel(log_level)
    echo_shandler.setFormatter(logging.Formatter(
        '[%(levelname)s]: %(message)s'))
    logger.addHandler(echo_shandler)

    # Errors and Critical errors handler (write to file)
    if error_log.lower() != 'none':
        warning_fhandler = logging.FileHandler(error_log)
        warning_fhandler.setLevel(logging.ERROR)
        warning_fhandler.setFormatter(logging.Formatter(
            '%(asctime)s [%(levelname)s]: %(message)s'))
        logger.addHandler(warning_fhandler)

    return logger


def retrieve_config(config_file, logger=None):
    """Retrieve configuration from YAML file.

    :param config_file: YAML configuration filename
    :type config_file: str
    :param logger: Logger to pass it to the ``Yamler`` constructor
    :type logger: logging.Logger
    :return: Dictionary of configuration
    :rtype: dict
    """
    config = Yamler(config_file, logger)

    site_config = {
        'uri': {
            'base': config.retrieve('base_uri', '').strip('/'),
            'canonical': config.retrieve('canonical_uri', '').rstrip('/'),
        },
        'wlocale': {
            'encoding': config.retrieve('encoding', 'utf-8'),
            'locale': config.retrieve('locale', 'en_US.UTF-8'),
            'language': config.retrieve('site_language', 'en'),
        },
        'date_format': {
            'entry': config.retrieve('datefmt_entry', "%c"),
            'home': config.retrieve('datefmt_home', "%b %_d, %Y"),
            'list': config.retrieve('datefmt_list', "%Y-%m-%d"),
        },
        'info': {
            'copyright': config.retrieve('site_copyright'),
            'site_author': config.retrieve('site_author', "Nameless"),
            'site_author_email': config.retrieve('site_author_email', ''),
            'site_description': config.retrieve('site_description'),
            'site_name': config.retrieve('site_name', "My Blog"),
        },
        'presentation': {
            'comments': config.retrieve('comments').lower() == "yes",
            'default_category':
                config.retrieve('default_category', "Miscellaneous"),
            'feed_format': config.retrieve('feed_format', "atom"),
            'max_entries': config.retrieve('max_entries', 10),
        },
        'dirs': {
            'deploy': "_build",
            'extra': config.retrieve('extra_dirs')
        }
    }

    return site_config


def arg_init(logger, dst):
    """Initialize a new website structure.

    :param dst: Name of the folder containing the new website
    :type dst: str
    :param logger: Logger to pass it to the ``Yamler`` constructor
    :type logger: logging.Logger
    :raise: OSError
    """
    real_path = os.path.dirname(os.path.realpath(__file__))
    src = os.path.join(real_path, 'initnew')
    dst = dst

    # create new blog structure with the default values
    try:
        shutil.copytree(src, dst)
    except OSError:
        logger and logger.error(
            "Unable to initialize the website structure")
        sys.exit(11)


def arg_build(logger, default_content_ext='.md',
              config_file='config.yml'):
    """Build the static website after getting the site configuration.

    :param logger: Logger to pass it to the ``Builder`` constructor
    :type logger: logging.Logger
    """
    site_config = retrieve_config(config_file, logger)

    # Prepare builder
    template_values = {
        'blog': {
            'author': site_config['info']['site_author'],
            'base_uri': site_config['uri']['base'],
            'encoding': site_config['wlocale']['encoding'],
            'feed_format': site_config['presentation']['feed_format'],
            'lang': site_config['wlocale']['language'],
            'site_name': site_config['info']['site_name'],
            'page_links': [],
        }
    }

    # Build
    b = Builder(site_config, template_values,
                infile_ext=default_content_ext, logger=logger)
    b.gen_site()


def arg_serve(logger, host='localhost', port=4000):
    """Initialize the server to listen until keyboard interruption.

    :param logger: Logger to pass it to the ``Server`` constructor
    :type logger: logging.Logger
    """
    server = Server(host, port=port, path='_build', logger=logger)
    server.serve()


def main():
    """Manage the command line arguments."""
    parser = argparse.ArgumentParser(description="PynFact!:"
                                     " A static blog generator from"
                                     " Markdown to HTML5 with Jinja2")
    group = parser.add_mutually_exclusive_group()

    group.add_argument('-i', '--init', default=None,
                       help="initialize a new blog structure")
    group.add_argument('-b', '--build', action='store_true',
                       help="build the website")
    group.add_argument('-s', '--serve', default='localhost',
                       help="set address to serve locally the blog")
    parser.add_argument('-p', '--port', default='4000', type=int,
                        help="set port to listen to when serving")
    parser.add_argument('-l', '--log', default='pynfact.err',
                        help="set a new error log filename (or 'none')")
    parser.add_argument('-v', '--verbose', action='store_true',
                        help="increase output verbosity")

    # Print help if no arguments supplied
    args = parser.parse_args(None if sys.argv[1:] else ['--help'])

    # Process arguments
    logger = set_logger(args.verbose, error_log=args.log)
    if args.init:
        arg_init(logger, args.init)
    elif args.build:
        arg_build(logger, default_content_ext='.md')
    elif args.serve:
        arg_serve(logger, args.serve, int(args.port))


# Main entry
if __name__ == '__main__':
    main()




# vim: set ft=python fileencoding=utf-8 tw=72 fdm=indent nowrap:
"""
Simple server for testing purposes.

    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""
from http.server import HTTPServer, SimpleHTTPRequestHandler
import os
import sys


class Server:
    """Simple server."""

    def __init__(self, host='127.0.0.1', port=4000, path='_build',
                 logger=None):
        """Constructor.

        :param host: Addres where the server will be listening
        :type host: str
        :param port: Port where the server will be listening
        :type port: str
        :param path: Where the static website will be generated
        :type path: str
        :param logger: Logger where to store activity in
        :type logger: logging.Logger
        """
        self.port = port
        self.host = host
        self.path = path
        self.logger = logger

    def serve(self):
        """Serve a specific directory and waits for keyboard interrupt.

        :raise: FileNotFoundError, OSError, KeyboardInterrupt
        """
        try:  # Find the deploy directory
            os.chdir(self.path)
        except FileNotFoundError:
            self.logger and self.logger.error("Deploy directory not found")
            sys.exit(61)

        try:  # Initialize the serve
            httpd = HTTPServer((self.host, self.port),
                               SimpleHTTPRequestHandler)
        except OSError:
            self.logger and self.logger.error(
                "Address not valid or already in use")
            sys.exit(62)

        self.logger and self.logger.info("Serving %s:%s at %s",
                                         self.host, self.port, self.path)

        try:  # Listen until a keyboard interruption
            httpd.serve_forever()
        except KeyboardInterrupt:
            self.logger and self.logger.info("Interrupted!")




# vim: set ft=python fileencoding=utf-8 tw=72 fdm=indent nowrap:
"""
Build the content.

Only markdown files will be taken, but markdown files are identified by
its extension, which for this application is ``.md`` (but it may be
specified in ``infile_ext`` argument)

    :copyright: © 2012-2020, J. A. Corbal
    :license: MIT
"""
from datetime import datetime
from feedgen.feed import FeedGenerator
from jinja2 import Environment, FileSystemLoader, Markup
from math import ceil
from pynfact.fileman import link_to, has_extension
from pynfact.meta import Meta
from pynfact.mulang import Mulang
from pynfact.struri import slugify, strip_html_tags
import distutils.dir_util
import filecmp
import gettext
import locale
import os
import sys
import textwrap


class Builder:
    """Site building process manager.

    .. todo:: Manage better with timezones. Make sure if there is no
        timezone, the default is always UTC, for posts info, and both
        feeds publication and modification dates.

    .. todo:: Remove redundant code using a universal method for
        retrieve and gather the data; and use subclasses.

    .. todo:: Unlink all file management from this class and build
        andother module to deal with that

    .. versionchanged:: 1.0.1.dev1
        Constructor takes a configuration dictionary instead of all
        parameters individually

    .. versionchanged:: 1.0.1a1
        Input pages stored in ``pages`` directory instead of root

    .. versionchanged:: 1.2.0a1
        Implement ``logging`` instead of printing to ``stdout`` and/or
        ``stderr``

    .. versionchanged:: 1.3.1.a2
        Update YAML loaded method to :func:``yaml.safe_load``

    """

    def __init__(self, site_config, template_values=dict(),
                 infile_ext='.md', logger=None):
        """Constructor.

        :param config: Site configuration as multidimensional dictionary
        :type config: dict
        :param template_values: Common values in all templates
        :type template_values: dict
        :param infile_ext: Posts files extension
        :type infile_ext: str
        :param logger: Logger where to store activity in
        :type logger: logging.Logger
        :raise: localeError
        """
        self.site_config = site_config
        self.template_values = template_values
        self.infile_ext = infile_ext.lower()
        self.site_config['dirs']['deploy'] = \
            os.path.join(self.site_config['dirs']['deploy'],
                         self.site_config['uri']['base'])
        self.logger = logger

        # Set locale for the site.
        self.old_locale = locale.getlocale()
        try:
            self.current_locale = \
                locale.setlocale(locale.LC_ALL,
                                 self.site_config['wlocale']['locale'])
        except locale.Error:
            self.logger and self.logger.error(
                "Unsupported locale setting")
            sys.exit(41)

        # Constants-like (I don't like this approach)
        # source dirs.
        self_dir = os.path.dirname(os.path.realpath(__file__))
        self.locale_dir = os.path.join(self_dir, 'locale')
        self.templates_dir = 'templates'
        self.builtin_templates_dir = \
            os.path.join(self.site_config['dirs']['deploy'],
                         self.templates_dir)

        # dest. dirs.
        self.home_cont_dir = 'page'         # Home paginator
        self.archive_dir = 'archive'        # Archive list page
        self.categories_dir = 'categories'  # One dir. per category
        self.tags_dir = 'tags'              # One dir. per tag

        # src. & dest. dirs.
        self.pages_dir = 'pages'            # Other pages such as 'About'...
        self.entries_dir = 'posts'          # Where posts are
        self.static_dir = 'static'          # CSS, JS, &c.

    def __del__(self):
        """Destructor.  Restore the locale, if changed."""
        return locale.setlocale(locale.LC_ALL, self.old_locale)

    def _render_template(self, template, output_data, values):
        """Render a template using Jinja2.

        :param template: Template to use
        :type template: str
        :param output_data: File where the data is saved
        :type output_data: str
        :return: Generated HTML of the output data
        :rtype: str
        """
        trans = gettext.translation('default', self.locale_dir,
                                    [self.current_locale])
        env = Environment(extensions=['jinja2.ext.i18n'],
                          loader=FileSystemLoader([self.templates_dir,
                                                   self.builtin_templates_dir]))
        env.install_gettext_translations(trans)
        env.globals['slugify'] = slugify  # Add `slugify` to Jinja2
        env.globals['strip_html_tags'] = strip_html_tags
        template = env.get_template(template)
        html = template.render(**values)

        # Update only those files that are different in content
        # comparing with a cache file
        with open(output_data + '~', mode="w",
                  encoding=self.site_config['wlocale']['encoding']) \
                as cache_file:
            cache_file.write(html)

        if not os.path.exists(output_data) or \
           not filecmp.cmp(output_data + '~', output_data):
            with open(output_data, mode="w",
                      encoding=self.site_config['wlocale']['encoding']) \
                    as output_file:
                output_file.write(html)
            self.logger and self.logger.info(
                'Updated content of: "%s"', output_data)

        # Clear cache, both in memory and space
        filecmp.clear_cache()
        os.remove(output_data + '~')

        return html

    def _entry_link_prefix(self, entry):
        """Compute entrie final path.

        To compute the final path, it's required to take the meta
        information of that entry concerning to title and publication
        date.
        Let's suppose the title is "My entry", and the date is
        "2020-04-01".  The output will be: `posts/2020/04/01/my-entry`

        :param: Entry filename
        :type: str
        :return: Path to this entry relative to root (slugified)
        :rtype: str
        """
        meta = Meta(Mulang(os.path.join(self.entries_dir, entry),
                           self.site_config['wlocale']['encoding'],
                           logger=self.logger).metadata())
        category = meta.category(
            self.site_config['presentation']['default_category'])
        date = meta.date('%Y-%m-%d')
        date_arr = date.split('-')
        path = os.path.join(str(self.entries_dir),
                            slugify(category),
                            str(date_arr[0]),
                            str(date_arr[1]),
                            str(date_arr[2]))
        return path

    def _make_entry_uri(self, name='', infix='', index='index.html',
                        force_root=False):
        """Generate the link to an entry, based on the date and name.

        :type name: Destination basename
        :param name: str
        :param infix: Appendix to the deploy directory
        :type infix: str
        :param index: Default name of the generated resource
        :type index: str
        :param force_root: Force the URI to be done from the root
        :type force_root: bool
        :return: Link to final external URI relative to root directory
        :rtype: str
        """
        if force_root:
            return link_to(name,
                           os.path.join('/',
                                        self.site_config['uri']['base'],
                                        self._entry_link_prefix(infix)),
                           makedirs=False, justdir=True, index=index)
        else:
            return link_to(name,
                           self._entry_link_prefix(infix),
                           makedirs=False, justdir=True, index=index)

    def _make_root_uri(self, name='', infix='', index='index.html'):
        """Generate the link to any resource in the root filesystem.

        :type name: Destination basename
        :param name: str
        :param infix: Appendix to the deploy directory
        :type infix: str
        :param index: Default name of the generated resource
        :type index: str
        :return: Link to final external URI relative to root directory
        """
        return link_to(name,
                       os.path.join('/',
                                    self.site_config['uri']['base'],
                                    infix),
                       makedirs=False, justdir=True, index=index)

    def _make_output_file(self, name='', infix='', index='index.html'):
        """Return the output file link, and make required directories.

        :param name: Destination basename
        :type name: str
        :param infix: Appendix to the deploy directory
        :type infix: str
        :param index: Default name of the generated resource
        :type index: str
        :return: Link to the constructed path in the deploy directory
        :rtype: str
        """
        return link_to(name,
                       os.path.join(self.site_config['dirs']['deploy'], infix),
                       makedirs=True, justdir=False, index=index)

    def gen_entry(self, infile, date_format='%c'):
        """Generate a HTML entry from its Mardkdown counterpart.

        :param infile: Markdown file to parse
        :type infile: str
        :param date_format: Date format for entry
        :type date_format: str
        :return: Generated HTML
        :rtype: str
        """
        inpath = os.path.join(self.entries_dir, infile)
        ml = Mulang(inpath, self.site_config['wlocale']['encoding'],
                    logger=self.logger)
        meta = Meta(ml.metadata())
        content_html = ml.html()

        title = meta.title()
        category = meta.category(
            self.site_config['presentation']['default_category'])
        timezone = meta.date('%z')
        if timezone:
            # to html5 timezone conv.
            timezone = timezone[:3] + ':' + timezone[3:]
        datehtml = meta.date('%Y-%m-%dT%H:%M') + timezone
        up_datehtml = meta.up_date('%Y-%m-%dT%H:%M') \
            + timezone if meta.up_date('%Y-%m-%dT%H:%M') \
                       else None

        values = self.template_values.copy()
        values['entry'] = {  # append
            'author': meta.author() \
            if meta.author() \
            else self.site_config['info']['site_author'],
            'title': title,
            'raw_title': strip_html_tags(title),
            'private': meta.private(),
            'comments': meta.comments(),
            'site_comments': self.site_config['presentation']['comments'],
            'category': category,
            'category_uri': self._make_root_uri(category,
                                                self.categories_dir),
            'date': meta.date(date_format),
            'datehtml': datehtml,
            'tags': meta.tag_list(),
            'content': content_html}

        # Publish original date if needed
        if meta.up_date():
            values['entry'].update({
                'up_date': meta.up_date(date_format),
                'up_datehtml': up_datehtml})

        outfile = self._make_output_file(
            title, self._entry_link_prefix(infile))
        return self._render_template('entry.html.j2', outfile, values)

    def gen_entries(self, date_format='%c'):
        """Generate all entries.

        :param date_format: Date format for entry
        :type date_format: str
        """
        for filename in os.listdir(self.entries_dir):
            if has_extension(filename, self.infile_ext):
                self.gen_entry(filename, date_format=date_format)

    def gen_home(self, max_entries_per_page=10, date_format='%Y-%m-%d'):
        """Generate home page, and subpages.

        :param max_entries: Max. entries per page
        :type max_entries: int
        :param date_format: Date format for home page
        :type date_format: str
        """
        entries = list()
        total_entries = 0
        for filename in os.listdir(self.entries_dir):
            if has_extension(filename, self.infile_ext):
                inpath = os.path.join(self.entries_dir, filename)
                meta = Meta(Mulang(inpath,
                                   self.site_config['wlocale']['encoding'],
                                   logger=self.logger).metadata())

                private = meta.private()
                title = meta.title()
                summary = meta.summary()
                category = meta.category(
                    self.site_config['presentation']['default_category'])
                category_uri = self._make_root_uri(category,
                                                   self.categories_dir)
                date = meta.date(date_format)
                date_idx = meta.date('%Y-%m-%d %H:%M:%S')
                uri = self._make_entry_uri(title, filename, force_root=True)

                # Generate archive
                if not private:
                    total_entries += 1
                    val = {'title': title, 'summary': summary,
                           'date': date, 'date_idx': date_idx,
                           'uri': uri, 'category': category,
                           'category_uri': category_uri}
                    entries.append(val)

        # Sort chronologically descent
        entries = sorted(entries, key=lambda k: k['date_idx'],
                         reverse=True)

        total_pages = ceil(total_entries / max_entries_per_page)
        values = self.template_values.copy()

        # FIXME :: Merge this scenario with the following loop
        # Generate 'index.html' even when there are no posts
        if total_entries == 0:
            outfile = self._make_output_file()
            print(outfile)
            self._render_template('entries.html.j2', outfile, values)

        # Home page (and subsequent ones)
        for cur_page in range(1, total_pages + 1):
            min_page = (cur_page - 1) * max_entries_per_page
            max_page = cur_page * max_entries_per_page

            values['entries'] = entries[min_page:max_page]
            values['cur_page'], values['total_pages'] = \
                cur_page, total_pages

            if cur_page == 1:
                # the home page, the "index.html" of the site
                outfile = self._make_output_file()
            else:
                # subsequent pages other that the first
                outfile = self._make_output_file(
                    str(cur_page), self.home_cont_dir)
            self._render_template('entries.html.j2', outfile, values)
            values['entries'] = {}  # reset the entries dict.

    def gen_archive(self, date_format='%c'):
        """Generate complete website archive, based on date.

        :param date_format: Date format for entry
        :type date_format: str
        """
        archive = dict()
        for filename in os.listdir(self.entries_dir):
            if has_extension(filename, self.infile_ext):
                post = os.path.join(self.entries_dir, filename)
                meta = Meta(Mulang(post,
                                   self.site_config['wlocale']['encoding'],
                                   logger=self.logger).metadata())

                private = meta.private()
                title = meta.title()
                #summary = meta.summary()
                category = meta.category(
                    self.site_config['presentation']['default_category'])
                date = meta.date(date_format)
                date_idx = meta.date('%Y-%m-%d')
                uri = self._make_entry_uri(title, filename)

                # Generate archive
                if not private:
                    val = {'uri': uri, 'title': title, 'date': date,
                           'date_idx': date_idx, 'category': category}
                    d = datetime.strptime(meta.date('%Y-%m-%d'), '%Y-%m-%d')
                    idx_year = d.strftime('%Y')
                    # %m to sort chronologically
                    idx_month = d.strftime('<%m> %B')
                    if idx_year in archive:
                        if idx_month in archive[idx_year]:
                            archive[idx_year][idx_month].append(val)
                        else:
                            archive[idx_year][idx_month] = [val]
                        # sort entries
                        archive[idx_year][idx_month] = \
                            sorted(archive[idx_year][idx_month],
                                   key=lambda k: k['date_idx'],
                                   reverse=False)
                    else:
                        archive[idx_year] = dict()
                        archive[idx_year][idx_month] = [val]

        values = self.template_values.copy()
        values['archive'] = archive
        outfile = self._make_output_file('', self.archive_dir)
        return self._render_template('archive.html.j2', outfile, values)

    def gen_category_list(self, date_format='%c'):
        """Generate categories page (an archive sorted by category).

        :param date_format: Date format for entry
        :type date_format: str
        """
        archive = dict()
        for filename in os.listdir(self.entries_dir):
            if has_extension(filename, self.infile_ext):
                post = os.path.join(self.entries_dir, filename)
                meta = Meta(Mulang(post,
                                   self.site_config['wlocale']['encoding'],
                                   logger=self.logger).metadata())

                private = meta.private()
                title = meta.title()
                #summary = meta.summary()
                category = meta.category(
                    self.site_config['presentation']['default_category'])
                date = meta.date(date_format)
                date_idx = meta.date('%Y-%m-%d')
                comments = meta.comments()
                uri = self._make_entry_uri(title, filename)

                # Generate archive
                if not private:
                    val = {'uri': uri, 'title': title, 'date': date,
                           'date_idx': date_idx, 'comments': comments}
                    if category in archive:
                        archive[category].append(val)
                    else:
                        archive[category] = [val]
                    # sort entries
                    archive[category] = sorted(archive[category],
                                               key=lambda k: k['date_idx'],
                                               reverse=True)

        values = self.template_values.copy()
        values['archive'] = archive
        outfile = self._make_output_file('', self.categories_dir)
        return self._render_template('catlist.html.j2', outfile, values)

    def gen_categories(self, date_format='%c'):
        """Generate categories pages.

        :param date_format: Date format for entry
        :type date_format: str
        """
        entries_by_category = dict()
        for filename in os.listdir(self.entries_dir):
            if has_extension(filename, self.infile_ext):
                post = os.path.join(self.entries_dir, filename)
                meta = Meta(Mulang(post,
                                   self.site_config['wlocale']['encoding'],
                                   logger=self.logger).metadata())

                private = meta.private()
                title = meta.title()
                #summary = meta.summary()
                category = meta.category(
                    self.site_config['presentation']['default_category'])
                date = meta.date(date_format)
                date_idx = meta.date('%Y-%m-%d')
                uri = self._make_entry_uri(title, filename)

                # Generate archive
                if not private:
                    val = {'uri': uri, 'title': title, 'date': date,
                           'date_idx': date_idx}
                    if category in entries_by_category:
                        entries_by_category[category].append(val)
                    else:
                        entries_by_category[category] = [val]

        # One page for each category
        values = self.template_values.copy()
        for category in entries_by_category:
            values['category_name'] = category
            values['entries'] = entries_by_category[category]
            # sort entries
            values['entries'] = sorted(values['entries'], key=lambda k:
                                       k['date_idx'], reverse=True)
            outfile = self._make_output_file(category, self.categories_dir)
            self._render_template('cat.html.j2', outfile, values)

    def gen_tags(self, date_format='%c'):
        """Generate tags pages.

        :param date_format: Date format for entry
        :type date_format: str
        """
        entries_by_tag = dict()
        for filename in os.listdir(self.entries_dir):
            if has_extension(filename, self.infile_ext):
                post = os.path.join(self.entries_dir, filename)
                meta = Meta(Mulang(post,
                                   self.site_config['wlocale']['encoding'],
                                   logger=self.logger).metadata())

                private = meta.private()
                title = meta.title()
                #summary = meta.summary()
                category = meta.category(
                    self.site_config['presentation']['default_category'])
                tag_list = meta.tag_list()
                date = meta.date(date_format)
                date_idx = meta.date('%Y-%m-%d')
                uri = self._make_entry_uri(title, filename)

                # Generate archive
                if not private:
                    val = {'uri': uri, 'title': title, 'date': date,
                           'date_idx': date_idx, 'category': category}
                    for tag in tag_list:
                        if tag in entries_by_tag:
                            entries_by_tag[tag].append(val)
                        else:
                            entries_by_tag[tag] = [val]

        # One page for each tag
        values = self.template_values.copy()
        for tag in entries_by_tag:
            values['tag_name'] = tag
            values['entries'] = entries_by_tag[tag]
            # sort entries
            values['entries'] = sorted(values['entries'], key=lambda k:
                                       k['date_idx'], reverse=True)
            outfile = self._make_output_file(tag, self.tags_dir)
            self._render_template('tag.html.j2', outfile, values)

    def gen_tag_cloud(self):
        """Generate tags cloud page.

        Tags will appear in different sizes depending on the their
        occurrences along the posts.  The more a tag is used, the bigger
        will be displayed.
        """
        entries_by_tag = dict()
        for filename in os.listdir(self.entries_dir):
            if has_extension(filename, self.infile_ext):
                post = os.path.join(self.entries_dir, filename)
                meta = Meta(Mulang(post,
                                   self.site_config['wlocale']['encoding'],
                                   logger=self.logger).metadata())
                private = meta.private()
                title = meta.title()
                tag_list = meta.tag_list()

                # Generate archive
                if not private:
                    val = {'title': title}
                    for tag in tag_list:
                        if tag in entries_by_tag:
                            entries_by_tag[tag].append(val)
                        else:
                            entries_by_tag[tag] = [val]

        # multipliers seq. for tag size in function of times repeated
        tagcloud_seq = [0, 14, 21, 27, 32, 38, 42, 45, 47, 48]

        # One page for each tag
        values = self.template_values.copy()
        values['tags'] = []
        for tag in entries_by_tag:
            if tag:
                tagname, tagfreq = tag, len(entries_by_tag[tag])
                mult = 100 + int(tagcloud_set.last
                                 if tagfreq > len(tagcloud_seq)
                                 else tagcloud_seq[tagfreq - 1])
                values['tags'].append({tagname: mult})

        outfile = self._make_output_file('', self.tags_dir)
        self._render_template('tagcloud.html.j2', outfile, values)

    def gen_nav_page_links(self):
        """Update the template data to contain also all page links.

        The navigation bar contains, not all links relevant to the site,
        but also all pages without the meta descriptor ``Navigate`` set
        to "no".

        .. note:: Since this add new data to the base template, it must
                  be invoked first, before generating any other page.
        """
        for filename in os.listdir(self.pages_dir):
            if has_extension(filename, self.infile_ext):
                inpath = os.path.join(self.pages_dir, filename)
                ml = Mulang(inpath,
                            self.site_config['wlocale']['encoding'],
                            logger=self.logger)
                meta = Meta(ml.metadata())

                if meta.navigation():
                    title = meta.title()
                    uri = self._make_root_uri(title)

                    values = self.template_values.copy()
                    values['page'] = {  # append
                        'title': title,
                        'raw_title': strip_html_tags(title),
                        'uri': uri}

                    # Update base template values to get pages links in
                    # nav. bar only if the page is set to be in the
                    # navigation bar
                    self.template_values['blog']['page_links'].append(
                        [values['page']['title'],
                         values['page']['uri']])

                    self.logger and self.logger.info(
                        'Added page to navigation: "%s"', inpath)

    def gen_page(self, infile):
        """Generate a HTML page from its Mardkdown counterpart.

        :param infile: Markdown file to parse
        :type infile: str
        :return: Generated HTML
        :rtype: str
        """
        inpath = os.path.join(self.pages_dir, infile)
        ml = Mulang(inpath, self.site_config['wlocale']['encoding'],
                    logger=self.logger)
        meta = Meta(ml.metadata())
        content_html = ml.html()

        title = meta.title()

        values = self.template_values.copy()
        values['page'] = {  # append
            'title': title,
            'raw_title': strip_html_tags(title),
            'content': content_html}

        outfile = self._make_output_file(title)
        return self._render_template('page.html.j2', outfile, values)

    def gen_pages(self):
        """Generate all pages."""
        for filename in os.listdir(self.pages_dir):
            if has_extension(filename, self.infile_ext):
                self.gen_page(filename)

    def gen_feed(self, feed_format="atom", outfile='feed.xml'):
        """Generate blog feed.

        :param feed_format: Feed format string ('rss' or 'atom').
                            If invalid value, will default to 'atom'
        :type feed_format: str
        :param outfile: Output filename
        :type outfile: str
        """
        feed = FeedGenerator()
        # feed.logo()
        feed.id(self.site_config['info']['site_name']
                if self.site_config['info']['site_name']
                else self.site_config['uri']['canonical'])
        feed.title(self.site_config['info']['site_name']
                   if self.site_config['info']['site_name']
                   else self.site_config['uri']['canonical'])
        feed.subtitle(self.site_config['info']['site_description']
                      if self.site_config['info']['site_description']
                      else 'Feed')
        feed.author({'name': self.site_config['info']['site_author'],
                     'email': self.site_config['info']['site_author_email']})
        feed.description(self.site_config['info']['site_description'])
        feed.link(href=os.path.join(self.site_config['uri']['canonical'],
                                    self.site_config['uri']['base']),
                  rel='alternate')
        feed.link(href=os.path.join(self.site_config['uri']['canonical'],
                                    self.site_config['uri']['base'],
                                    outfile), rel='self')
        feed.language(self.site_config['wlocale']['language'])
        feed.copyright(self.site_config['info']['copyright'])

        entries = list()
        for filename in os.listdir(self.entries_dir):
            if has_extension(filename, self.infile_ext):
                infile = os.path.join(self.entries_dir, filename)
                ml = Mulang(infile,
                            self.site_config['wlocale']['encoding'],
                            logger=self.logger)
                meta = Meta(ml.metadata())
                author = self.site_config['info']['site_name'] \
                    if not meta.author() \
                    else meta.author()
                email = self.site_config['info']['site_author_email'] \
                    if not meta.email() \
                    else meta.email()
                content_html = ml.html()
                private = meta.private()
                title = meta.title()
                summary = meta.summary()
                up_date_idx = meta.up_date('%Y-%m-%d %H:%M:%S')
                date_idx = meta.date('%Y-%m-%d %H:%M:%S')
                timezone = 'UTC' if not meta.date('%Z') else meta.date('%Z')
                date_iso8601 = meta.date('%Y-%m-%dT%H:%M') + timezone
                up_date_iso8601 = meta.up_date('%Y-%m-%dT%H:%M') \
                    + timezone if meta.up_date('%Y-%m-%dT%H:%M') \
                               else None
                pub_date = date_iso8601
                up_date = up_date_iso8601

                #updated = datetime.strptime(date_iso8601, '%Y-%m-%dT%H:%M%Z')
                uri = self._make_entry_uri(title, filename)
                full_uri = os.path.join(self.site_config['uri']['canonical'],
                                        self.site_config['uri']['base'], uri)

                if not private:
                    val = {'title': strip_html_tags(title),
                           'subtitle': summary,
                           'content_html': content_html,
                           'author': author,
                           'email': email,
                           'full_uri': full_uri,
                           'date_idx': date_idx,
                           'pub_date': pub_date,
                           'up_date': up_date}
                    entries.append(val)

        # sort chronologically descent
        entries = sorted(entries, key=lambda k: k['date_idx'],
                         reverse=True)
        for entry in entries:
            fnew = feed.add_entry()
            fnew.id(slugify(strip_html_tags(entry['title'])))
            fnew.title(entry['title'])
            fnew.description(entry['content_html'])
            if up_date:
                fnew.updated(entry['up_date'])
            fnew.pubDate(entry['pub_date'])
            # , 'email':entry['email']})
            fnew.author({'name': entry['author']})
            fnew.link(href=entry['full_uri'], rel='alternate')

        if feed_format == "rss":
            feed.rss_file(os.path.join(self.site_config['dirs']['deploy'],
                                       outfile))
        else:
            feed.atom_file(os.path.join(self.site_config['dirs']['deploy'],
                                        outfile))

    def gen_static(self):
        """Generate (copies) static directory."""
        src = self.static_dir
        dst = os.path.join(self.site_config['dirs']['deploy'],
                           self.static_dir)
        if os.path.exists(src):
            distutils.dir_util.copy_tree(src, dst, update=True,
                                         verbose=True)

    def gen_extra_dirs(self):
        """Generate extra directories if they exist."""
        if self.site_config['dirs']['extra']:
            for extra_dir in self.site_config['dirs']['extra']:
                src = extra_dir
                dst = os.path.join(self.site_config['dirs']['deploy'],
                                   extra_dir)
                if os.path.exists(src):
                    distutils.dir_util.copy_tree(src, dst, update=True,
                                                 verbose=True)

    def gen_site(self):
        """Generate all website content.

        .. note:: The first thing that has to be generated is the
                  navigation links for all user defined pages.
                  Otherwise those links could be left behind on page
                  pages.
        """
        self.logger and self.logger.info('Building static website...')

        self.gen_nav_page_links()
        self.gen_entries(self.site_config['date_format']['entry'])
        self.gen_pages()
        self.gen_archive(self.site_config['date_format']['list'])
        self.gen_categories(self.site_config['date_format']['list'])
        self.gen_category_list(self.site_config['date_format']['list'])
        self.gen_tags(self.site_config['date_format']['list'])
        self.gen_tag_cloud()
        self.gen_home(self.site_config['presentation']['max_entries'],
                      self.site_config['date_format']['home'])
        self.gen_feed(self.site_config['presentation']['feed_format'])
        self.gen_static()
        self.gen_extra_dirs()

        self.logger and self.logger.info('Done!')




#!/usr/bin/env python3
# vim: set ft=python fileencoding=utf-8 tw=72 nowrap:

from setuptools import setup, find_packages


version = '1.3.1a2'

with open("README.rst", "r") as fh:
    long_description = fh.read()


setup(name='pynfact',
      version=version,
      author='J. A. Corbal',
      author_email='jacorbal@gmail.com',
      url='https://github.com/jacorbal/pynfact/wiki',
      download_url='https://github.com/jacorbal/pynfact',
      project_urls={
          'Documentation': 'https://github.com/jacorbal/pynfact/wiki',
          'Funding': 'https://jacorbal.es/pynfact',
          'Source': 'https://github.com/jacorbal/pynfact',
          'Tracker': 'https://github.com/jacorbal/pynfact/issues'},
      description='Blog-oriented static website generator using Markdown.',
      long_description=long_description,
      long_description_content_type='text/x-rst',
      license='MIT',
      keywords=['blog', 'markdown', 'static', 'web', 'site', 'generator'],
      py_modules=find_packages(),
      packages=['pynfact'],
      entry_points={'console_scripts': ['pynfact = pynfact.__main__:main']},
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: End Users/Desktop',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Topic :: Internet :: WWW/HTTP',
          'Topic :: Internet :: WWW/HTTP :: Site Management',
          'Topic :: Software Development :: Code Generators',
          'Topic :: Text Processing :: Markup :: HTML',
          'Topic :: Utilities',
      ],
      install_requires=[
          'feedgen >= 0.9.0',
          'jinja2 >= 2.7',
          'markdown >= 3.0.0',
          'pygments',
          'pyyaml >= 5.1',
          'unidecode',
      ],
      python_requires='>=3.6',
      include_package_data=True,
      )

